﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Infragistics.Win.UltraWinTree;
using Infragistics.Win;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private DataSet dsReleases = new DataSet();

        public Form1()
        {
            InitializeComponent();

            FillTables();
            FillTree();
        }

        private void FillTables()
        {
            DataTable dtReleases = new DataTable("Releases");

            DataColumn dcSelect = new DataColumn("Select", typeof(int));
            dtReleases.Columns.Add(dcSelect);

            DataColumn dcRelease = new DataColumn("ReleaseID", typeof(Int64));
            dtReleases.Columns.Add(dcRelease);       

            for(int i = 0; i<5; i++)
            {
                DataRow dr = dtReleases.NewRow();
                dr["Select"] = 0;
                dr["ReleaseID"] = i;

                dtReleases.Rows.Add(dr);              
            }

            dtReleases.AcceptChanges();
            dsReleases.Tables.Add(dtReleases);
            dsReleases.AcceptChanges();
        }

        private void FillTree()
        {
            utReleases.BeginUpdate();

            utReleases.Indent = 10;
            utReleases.SetDataBinding(null, string.Empty);
            utReleases.Nodes.Clear();
            utReleases.ColumnSettings.Reset();

            //utReleases.ViewStyle = Infragistics.Win.UltraWinTree.ViewStyle.OutlookExpress;
            utReleases.Override.ShowExpansionIndicator = ShowExpansionIndicator.CheckOnDisplay;
            utReleases.Override.Sort = SortType.None;
            utReleases.ColumnSettings.AllowSorting = DefaultableBoolean.False;
            utReleases.ColumnSettings.ColumnAutoSizeMode = Infragistics.Win.UltraWinTree.ColumnAutoSizeMode.AllNodesWithDescendants;
            utReleases.ColumnSettings.AutoFitColumns = Infragistics.Win.UltraWinTree.AutoFitColumns.None;

            utReleases.SetDataBinding(dsReleases, dsReleases.Tables[0].TableName);

            UltraTreeColumnSet pColSet = utReleases.Nodes.ColumnSetResolved;

            pColSet.Columns["Select"].Text = "Select";
               
            pColSet.Columns["Select"].Editor = new EditorWithText();
            pColSet.Columns["Select"].EditorResolved.DataFilter = new CheckEditorDataFilter();
            pColSet.Columns["Select"].EditorResolved.ValueChanged += new EventHandler(pEditor_ValueChanged);
            pColSet.Columns["Select"].CellAppearance.ImageHAlign = HAlign.Center;

            utReleases.PerformLayout();
            utReleases.EndUpdate();
            Cursor = Cursors.Default;
        }

        // For Infragistics Support, I commented out/removed a bunch of this method, the main problem that I see is utReleases.ActiveNode.Nodes causes
        // utReleases_AfterDataNodesCollectionPopulated to fire.
        private void pEditor_ValueChanged(object sender, EventArgs e)
        {
            if (utReleases.ActiveNode == null) return;
            EditorWithText pEditor = (EditorWithText)sender;
            if (utReleases.ActiveNode.Parent == null)
            {
                foreach (UltraTreeNode pNode in utReleases.ActiveNode.Nodes)
                {
                    //Do stuff with node.
                }
            }
        }

        private void CheckSelectedNode(UltraTreeNode pNode, string cellName, CheckState checkValue)
        {
            switch (checkValue)
            {
                case CheckState.Checked:
                    if (pNode.Cells.Exists(cellName))
                        pNode.Cells[cellName].Appearance.Image = Properties.Resources.CheckBox;
                  
                    break;

                case CheckState.Unchecked:
                    if (pNode.Cells.Exists(cellName))
                        pNode.Cells[cellName].Appearance.Image = Properties.Resources.uncheckbox;                   

                    break;

                case CheckState.Indeterminate:
                    if (pNode.Cells.Exists(cellName))
                        pNode.Cells[cellName].Appearance.Image = Properties.Resources.checkboxTriState;

                    break;
            }
        }

        private void utReleases_AfterDataNodesCollectionPopulated(object sender, AfterDataNodesCollectionPopulatedEventArgs e)
        {
            try
            {
                foreach (UltraTreeNode pNode in utReleases.Nodes)
                {
                    pNode.Cells["Select"].Value = 0;
                    CheckSelectedNode(pNode, "Select", CheckState.Unchecked);
                    if (pNode.HasNodes)
                    {
                        foreach (UltraTreeNode cNode in pNode.Nodes)
                        {
                            cNode.Cells["Select"].Value = 0;
                            CheckSelectedNode(cNode, "Select", CheckState.Unchecked);
                        }
                    }
                }
            }
            catch(Exception ex) 
            {
                MessageBox.Show( ex.ToString());
            }
        }

        private void utReleases_Click(object sender, EventArgs e)
        {
            if (utReleases.SelectedNodes.Count < 1) return;
            if (utReleases.ActiveCell == null) return;
            switch (utReleases.ActiveCell.Column.Key)
            {
                case "Select":
                    utReleases.SelectedNodes[0].Cells["Select"].AllowEdit = AllowCellEdit.Full;
                    utReleases.SelectedNodes[0].Cells["Select"].BeginEdit();
                    EditorWithText pEditor = (EditorWithText)(utReleases.Nodes.ColumnSetResolved.Columns["Select"].EditorResolved);

                    if ((CheckState)utReleases.ActiveCell.Value == CheckState.Unchecked)
                        utReleases.ActiveCell.Value = CheckState.Checked;
                    else
                        utReleases.ActiveCell.Value = CheckState.Unchecked;

                    bool bChecked = (int)utReleases.ActiveCell.Value != 0;
                    CheckSelectedNode(utReleases.SelectedNodes[0], "Select", (CheckState)utReleases.ActiveCell.Value);
                    utReleases.SelectedNodes[0].Update();

                    utReleases.SelectedNodes[0].Cells["Select"].AllowEdit = AllowCellEdit.ReadOnly;
                    if (!utReleases.ActiveCell.Node.Expanded && bChecked)
                        utReleases.ActiveCell.Node.ExpandAll();
                    else if (utReleases.ActiveCell.Node.Expanded && !bChecked)
                        utReleases.ActiveCell.Node.CollapseAll();
                    utReleases.ActiveCell = null;
                    break;                
            }
        }
    }

    public class CheckEditorDataFilter : Infragistics.Win.IEditorDataFilter
    {
        object Infragistics.Win.IEditorDataFilter.Convert(Infragistics.Win.EditorDataFilterConvertArgs args)
        {
            switch (args.Direction)
            {
                case ConversionDirection.EditorToOwner:
                    args.Handled = true;
                    if (args.Value == "")
                        return CheckState.Unchecked;
                    else if (args.Value == " ")
                        return CheckState.Checked;
                    else
                        return CheckState.Indeterminate;

                //case ConversionDirection.EditorToDisplay:
                case ConversionDirection.OwnerToEditor:
                    args.Handled = true;
                    if (args.Value == DBNull.Value)
                        return "   ";
                    if ((CheckState)args.Value == CheckState.Unchecked)
                        return "";
                    else if ((CheckState)args.Value == CheckState.Checked)
                        return " ";
                    else
                        return "  ";

                default:
                    return args.Value;
            }
            throw new Exception("ERROR - CheckEditorDataFilter.Convert()");
        }
    }
}
